import csv
import re

from jinja2 import Environment, PackageLoader

try:
    import colorbrewer
except ImportError:
    default_brewer = None
else:
    default_brewer = colorbrewer.Spectral

_env = Environment(loader=PackageLoader(__package__, 'data'))


def regex_replace(s, pattern, replace):
    """ Enable regex replacement in jinja template """
    _re = re.compile(pattern)
    return _re.sub(replace, s)


_env.filters['regex_replace'] = regex_replace


def rgb_from_triplet(triplet):
    return "#" + format((triplet[0] << 16) | (triplet[1] << 8) | triplet[2], '06x')


def load_template():
    """ Loads the template file, ready for rendering. """
    return _env.get_template('html_template.jinja')


def print_html(formatted_seqs, vnumstrs, mask, outfile, annotations=None,
               fontsize=12, seqnums=None, charcolors=None, tableonly=False):
    # XXX - Uhh... what are seqnums?
    render_dict = dict(seqlist=formatted_seqs, vnumstrs=vnumstrs,
                       mask=mask, annotations=annotations, colors=charcolors,
                       tableonly=tableonly)
    with open(outfile, 'w') as handle:
        load_template().stream(render_dict).dump(handle)


def parse_character_color_file(char_color_file):
    """ Parses file into a dict map of characters to css colors """
    reader = csv.DictReader(char_color_file, fieldnames=['name', 'color'])
    mapping = dict()
    for row in reader:
        mapping[row['name']] = row['color']

    if hasattr(char_color_file, 'close'):
        char_color_file.close()

    return mapping


def parse_mapping_file(mapping_handle):
    """ Parses into a dict map of col numbers to groups """
    reader = csv.DictReader(mapping_handle)
    mapping = dict()
    for row in reader:
        # later will want to support col formats like 23-45
        mapping[int(row['col'])] = row['group']

    mapping_handle.close()

    return mapping


class AnnotationSet(object):

    @classmethod
    def from_mapping_file(cls, handle, mask, color_mapping=None,
                          brewer=default_brewer):
        """ Convenience method for loading directly from files. """
        mapping = parse_mapping_file(handle)
        return cls(mapping, mask, color_mapping)

    def __init__(self, col_mapping, mask, color_mapping=None, brewer=default_brewer):
        """col_mapping is a mapping of 1-index based columns to group
        names. mask is the mask to be applied to sequences and number
        string rows.

        """
        self.col_mapping = col_mapping
        self.mask = mask
        self.mask_cols = [i + 1 for i in range(len(mask)) if mask[i]]
        self.groups = list(
            set(self.col_mapping[c] for c in list(self.col_mapping.keys())))

        if color_mapping:
            self.color_mapping = color_mapping
        else:
            n = len(self.groups)
            try:
                pallette = brewer[n]
            except KeyError:
                pallette = brewer[min(brewer.keys())]

            colors = [rgb_from_triplet(t) for t in pallette]
            self.color_mapping = dict()
            for i in range(n):
                self.color_mapping[self.groups[i]] = colors[i]

    def masked_index(self, col):
        return self.mask_cols.index(col)

    def get_color(self, group):
        if group is None:
            return None
        return self.color_mapping[group]

    def iterate(self, iterable):
        """ Iterate over a seq or number string """
        latest_region = []
        last_group = None

        def join_region(l):
            return ''.join(str(x) for x in l)

        for i in range(len(self.mask)):
            col = i + 1
            if not self.mask[i]:
                # If masked, we just skip
                continue

            try:
                group = self.col_mapping[col]
            except KeyError:
                group = None

            index = self.masked_index(col)
            item = iterable[index]
            if group == last_group:
                latest_region.append(item)
                last_yield = False
                # import pdb; pdb.set_trace()

            else:
                last_yield = True
                yield (join_region(latest_region),
                       last_group,
                       self.get_color(last_group))
                latest_region = [item]
                last_group = group

        if not last_yield:
            # Hmm... do we actually need the check here?
            yield (join_region(latest_region), last_group, self.get_color(last_group))



import signal
import sys
import glob
import os

from ._version import get_version

_data = os.path.join(os.path.dirname(__file__), 'data')
__version__ = get_version()


def _exit_on_signal(sig, status=None, message=None):
    def exit(sig, frame):
        if message:
            sys.stderr.write(message)
        raise SystemExit(status)
    signal.signal(sig, exit)


def exit_on_sigint(status=1, message="Canceled."):
    """
    Set program to exit on SIGINT, with provided status and message.
    """
    _exit_on_signal(signal.SIGINT, status, message)


def exit_on_sigpipe(status=None):
    """
    Set program to exit on SIGPIPE
    """
    _exit_on_signal(signal.SIGPIPE, status)


def package_data(fname, pattern=None, data_dir=_data):
    """Return the absolute path to a file included in package data,
    raising ValueError if no such file exists. If `pattern` is
    provided, return a list of matching files in package data
    (ignoring `fname`).

    """

    if pattern:
        return glob.glob(os.path.join(data_dir, pattern))

    pth = os.path.join(data_dir, fname)

    if not os.path.exists(pth):
        raise ValueError('Package data does not contain the file %s' % fname)

    return pth



import re
import math
import itertools
from collections import Counter, namedtuple

from fastalite import fastalite

Seq = namedtuple('Seq', ['name', 'seq'])

SIMCHAR = '.'
GAPCHAR = '-'
ERRCHAR = 'X'

try:
    from Bio import Phylo
except ImportError:
    treeorder = None
else:
    def treeorder(infile):
        tree = next(Phylo.parse(infile, 'newick'))
        tree.ladderize()
        return [leaf.name for leaf in tree.get_terminals()]


def get_seq_from_list(compare_to, seqlist):
    """
    Return an item from 'seqlist' identified by string 'compare_to', defines as either:

    - a substring matching the name of a single sequence (case-sensitive)
    - the 1-based index if > 0
    - the 0-based index if < 1

    Raises ValueError if 'compare_to' doesn't meet one of the above criteria.
    """

    # assume that compare_to is the index of a sequence if it can be
    # coerced to an int, and the resulting index is in the range of
    # len(seqlist)
    try:
        ref_ix = int(compare_to)
        # assumes 1-based index if positive, or 0-based offset from
        # end if 0 or negative
        _s = seqlist[ref_ix - 1] if ref_ix > 0 else seqlist[ref_ix]
    except (ValueError, IndexError):
        pass
    else:
        return _s

    # doesn't seem to be an index... is it a string uniquely matching
    # a sequence name?
    matches = [i for i, seq in enumerate(seqlist) if compare_to in seq.name]
    if len(matches) == 1:
        return seqlist[matches[0]]

    # getting this far indicates an error
    raise ValueError(
        '"{}" must be either a string matching the name of a *single* sequence, '
        'or 1-based index of a sequence')

    return _s


def get_extent(seqstr):
    """Return (start, end) identifying the 0-based extent of seqstr
    excluding end-gaps.

    """

    start = re.search(r'[a-z]', seqstr, flags=re.I).start()
    stop = len(re.sub(r'[^a-z]+$', '', seqstr, flags=re.I))
    return start, stop


def reformat(seqs,
             add_consensus=True,
             compare=True,
             compare_to=None,
             exclude_gapcols=True,
             exclude_invariant=False,
             min_subs=1,
             simchar=SIMCHAR,
             gapchar=GAPCHAR,
             count_gaps=False,
             seqrange=None,
             trim_to=None,
             reference_top=False):
    """
    Reformat an alignment of sequences for display. Return a list of
    lists of strings; the outer list corresponds to pages.

    * seqs - list of objects with attributes 'name' and 'seq'
    * add_consensus - If True, include consensus sequence.
    * compare - if True (default), compare each character to
      corresponding position in the sequence specified by `compare_to`
      and replace with `simchar` if identical.
    * compare_to - Name or 1-based index of a reference sequence. None
      (the default) specifies the consensus. Ignored if `compare` is False.
    * exclude_gapcols - if True, mask columns with no non-gap characters
    * exclude_invariant - if True, mask columns without minimal polymorphism
    * min_subs -
    * simchar - character indicating identity to corresponding position in compare_to
    * gapchar - character representing gaps
    * count_gaps - include gaps in calculation of consensus and columns to display
    * seqrange - optional two-tuple specifying start and ending
      coordinates (1-index, inclusive)
    * trim_to - trim the alignment to the start and end positions of
      this sequence identified by name or 1-based index (ignored if
      seqrange is provided).
    * seqnums - show sequence numbers (1-index) to left of name if True
    * reference_top - put reference/consensus sequence at top instead of bottom
    """

    seqlist = list(seqs)
    nseqs = len(seqlist)

    # a list of Counter objects
    tabulated = tabulate(seqlist)

    cons = Seq(
        name='CONSENSUS',
        seq=''.join([consensus(d, count_gaps=count_gaps) for d in tabulated])
    )

    if add_consensus:
        if reference_top:
            seqlist.insert(0, cons)
        else:
            seqlist.append(cons)

    if compare:
        # replace bases identical to reference; make a copy of the
        # sequence for comparison because the original sequences will
        # be modified.
        if compare_to:
            compare_to_name, compare_to_str = get_seq_from_list(compare_to, seqlist)
        else:
            compare_to_name, compare_to_str = cons

        # replace characters identical to the reference
        for i, seq in enumerate(seqlist):
            name, seqstr = seq
            if name == compare_to_name:
                name = '==REF==> ' + name
            else:
                seqstr = seqdiff(seqstr, compare_to_str, simchar=simchar)

            seqlist[i] = Seq(name, seqstr)

    if seqrange:
        start, stop = seqrange
        start -= 1  # seqrange is 1-indexed
    elif trim_to:
        start, stop = get_extent(get_seq_from_list(trim_to, seqlist).seq)
    else:
        start, stop = 0, len(tabulated)

    mask = []
    for i, counts in enumerate(tabulated):
        show = start <= i < stop

        if exclude_gapcols:
            show &= counts.get(gapchar, 0) < nseqs

        if exclude_invariant:
            show &= count_subs(counts, count_gaps=count_gaps) >= min_subs

        mask.append(show)

    vnumstrs = [apply_mask(s, mask) for s in get_vnumbers(cons.seq)]
    seqlist = [Seq(s.name, apply_mask(s.seq, mask)) for s in seqlist]

    return (seqlist, vnumstrs, mask)


def apply_mask(instr, mask):
    return ''.join(c for c, m in zip(instr, mask) if m)


def pagify(seqlist, vnumstrs,
           name_min=10,
           name_max=35,
           nrow=65,
           ncol=70,
           all_numstrs=True,
           seqnums=False):
    """ This does the work of taking the mostly formatted sequences
    (still as seqobjs) and joining them together with names for the
    pdf and stdout outputs."""

    # XXX - todo, improve docs here ^

    seqcount = len(seqlist)

    longest_name = max([len(s.name) for s in seqlist])
    name_width = max([name_min, min([longest_name, name_max])])

    num_width = math.floor(math.log10(seqcount)) + 1

    fstr = '%%(name)%(name_width)ss %%(seqstr)-%(ncol)ss' % locals()
    if seqnums:
        fstr = ('%%(count)%(num_width)is ' % locals()) + fstr

    colstop = len(seqlist[0].seq)

    out = []
    # start is leftmost column for each block of columns
    for start in range(0, colstop, ncol):
        stop = min([start + ncol, colstop])

        # breaks into vertical blocks of sequences
        counter = itertools.count(1)
        for first in range(0, seqcount, nrow):
            out.append([])
            last = min([first + nrow, seqcount])

            msg = ''
            if seqcount > nrow:
                msg += 'sequences %s to %s of %s' % (first + 1, last, seqcount)

            if msg:
                out[-1].append(msg)

            this_seqlist = seqlist[first:last]

            if all_numstrs:
                # label each position
                for s in vnumstrs:
                    out[-1].append(
                        fstr % {'count': '', 'name': '#', 'seqstr': s[start:stop]})
            else:
                # label position at beginning and end of block
                half_ncol = int((stop - start) / 2)
                numstr = ' ' * name_width + \
                    ' %%-%(half_ncol)ss%%%(half_ncol)ss\n' % locals()
                out[-1].append(numstr % (start + 1, stop))

            for seq in this_seqlist:
                count = next(counter)
                seqstr = seq.seq[start:stop]
                name = seq.name[:name_width]
                out[-1].append(fstr % locals())

    return out


def readfasta(infile, name_split=None):
    """Returns an iterator of namedtuple objects with attributes 'name'
    and 'seq'.  'name_split' is a string on which to split sequence
    names, or "none" to define seq.name as the entire header line.

    """

    seqs = fastalite(infile)
    if name_split == 'none':
        seqs = (Seq(seq.description, seq.seq) for seq in seqs)
    elif name_split:
        seqs = (Seq(seq.description.split(name_split)[0], seq.seq) for seq in seqs)
    else:
        seqs = (Seq(seq.id, seq.seq) for seq in seqs)

    return seqs


def tabulate(seqs):
    """Return a list of Counter() objects describing the base frequency
    at each position in seqs.

    """

    seqs = list(seqs)
    lengths = {len(seq.seq) for seq in seqs}
    if len(lengths) > 1:
        raise ValueError('all sequences must be the same length')

    seqlen = lengths.pop()
    counters = [Counter() for i in range(seqlen)]
    for seq in seqs:
        for i, c in enumerate(seq.seq.upper()):
            counters[i].update([c])

    return counters


def consensus(count, count_gaps=False, plurality=2, gapchar='-', errchar='X'):
    """Calculates the consensus chacater at a position.

    * count - a Counter() object from tabulate() representing
      character frequencies at a single position
    * count_gaps - boolean indicating whether to include gap characters
      in tabulation of the consensus character.
    * plurality - minimum difference in frequency between most common and
      second most common characters
    * gapchar - the character representing a gap
    * errchar - character representing a position at which consensus
      could not be calculated (ie, count_gaps is False and all
      characters are gaps, or the difference in frequency between the
      first and second most common characaters is < plu)

    """

    if not count_gaps:
        try:
            del count[gapchar]
        except KeyError:
            pass

    if len(count) == 0:
        conschar = errchar
    elif len(count) == 1:
        conschar = count.most_common(1)[0][0]
    else:
        (char_1, count_1), (char_2, count_2) = count.most_common(2)
        if count_1 - count_2 < plurality:
            conschar = errchar
        else:
            conschar = char_1

    return conschar


def count_subs(tabdict, count_gaps=False, gap='-'):
    """Given a dict representing character frequency at a
    position of an alignment, returns 1 if more than
    one character is represented, 0 otherwise; excludes gaps
    if not count_gaps. There must be at least minchars present for
    a position to be considered variable"""

    tabdict = tabdict.copy()

    if not count_gaps:
        try:
            del tabdict[gap]
        except KeyError:
            pass

    if len(tabdict) <= 1:
        return 0

    # calculate count of most frequent character
    vals = sorted(tabdict.values())
    total = sum(vals)
    majority_char_count = vals.pop(-1)

    substitutions = total - majority_char_count

    return substitutions


def seqdiff(seq, templateseq, simchar=SIMCHAR, gapchar=GAPCHAR):
    """Compares strings seq and templateseq and returns a string in which
    non-gap characters in seq that are identical at that position to
    templateseq are replaced with simchar. Returned string is the
    length of the shorter of seq and templateseq.

    """

    if simchar and len(simchar) > 1:
        raise ValueError('simchar must contain a single character only')

    if simchar:
        seqstr = seq.upper()
        templatestr = templateseq.upper()

        def diff(s, t):
            return simchar if s == t and s != gapchar else s
    else:
        seqstr = seq.lower()
        templatestr = templateseq.lower()

        def diff(s, t):
            return s.upper() if s == t else s

    return ''.join(diff(s, t) for s, t in zip(seqstr, templatestr))


def get_vnumbers(seqstr, ignore_gaps=True, leadingZeros=True, gapchar=GAPCHAR):

    seqlen = len(seqstr)

    digs = len(repr(seqlen + 1))
    if leadingZeros:
        fstr = '%%0%si' % digs
    else:
        fstr = '%%%ss' % digs

    gapstr = gapchar * digs

    numchars = []
    i = 1
    for c in seqstr:
        if not ignore_gaps and c == gapchar:
            numchars.append(gapstr)
        else:
            numchars.append(fstr % i)
            i += 1

    if ignore_gaps:
        assert numchars == [fstr % x for x in range(1, seqlen + 1)]

    return [''.join([x[n] for x in numchars]) for n in range(digs)]



import os
from itertools import takewhile

try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest


def grouper(n, iterable, pad=True):
    """
    Return sequence of n-tuples composed of successive elements
    of iterable; last tuple is padded with None if necessary. Not safe
    for iterables with None elements.
    """

    args = [iter(iterable)] * n
    iterout = zip_longest(fillvalue=None, *args)

    if pad:
        return iterout
    else:
        return (takewhile(lambda x: x is not None, c) for c in iterout)

try:
    from reportlab.pdfgen import canvas
    from reportlab.lib.units import inch, cm
    from reportlab.lib.pagesizes import letter, landscape, portrait
except ImportError:
    print_pdf = None
else:
    def print_pdf(pages, outfile, fontsize = 7, orientation='portrait',
                  blocks_per_page=1):

        if orientation == 'portrait':
            pagesize = portrait(letter)
        elif orientation == 'landscape':
            pagesize = landscape(letter)

        #precalculate some basics
        top_margin = pagesize[1] - .75*inch
        bottom_margin = 0.75*inch
        left_margin = 0.5*inch
        right_margin = pagesize[0] - 0.5*inch
        frame_width = right_margin - left_margin

        top_margin_offset = 0

        document_font = 'Courier'

        def drawPageFrame(canv):

            # pagecount = len(pages)
            pagecount = len(pageIter)
            pg = canv.getPageNumber()
            ofn = os.path.split(outfile)[-1]

            page_center = 0.5 * pagesize[0]

            canv.drawCentredString(page_center, top_margin + 0.3*inch,
                '%(ofn)s' % locals())

            lineycoord = top_margin + 0.25*inch
            canv.line(left_margin, lineycoord, right_margin, lineycoord)
    #       canv.setFont(document_font,fontsize)

            canv.drawCentredString(page_center, 0.5*inch,
                'Page %(pg)s of %(pagecount)s' % locals())

        canv = canvas.Canvas(outfile, invariant=1, pagesize=pagesize)
        canv.setPageCompression(1)

        pageIter = list(grouper(blocks_per_page, pages, pad=False))

        for pagegroup in pageIter:
            drawPageFrame(canv)
            canv.setFont(document_font, fontsize)
            tx = canv.beginText(left_margin, top_margin - top_margin_offset)

            for i, page in enumerate(pagegroup):
                for line in page:
                    tx.textLine(line)
                # separate blocks
                if blocks_per_page > 1 and i + 1 < blocks_per_page:
                    tx.textLine('')

            canv.drawText(tx)
            canv.showPage()

        canv.save()



#!/usr/bin/env python

"""
Create formatted sequence alignments with optional pdf output.
"""

import sys
import argparse
import csv

from alnvu import util, pdf, html, package_data
from alnvu import __version__, exit_on_sigint, exit_on_sigpipe


def get_range(rawrange):

    try:
        start, stop = [int(x) for x in rawrange.split(',')]
    except (ValueError, AttributeError):
        print((
            'Error in "-r/--range {}": argument requires two '
            'integers separated by a comma.'.format(rawrange)))
        sys.exit(1)

    return [start, stop]


def main(arguments=None):

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '-V', '--version', action='version', version='alnvu {}'.format(__version__))

    parser.add_argument(
        "infile", type=argparse.FileType('r'), nargs='?',
        default=sys.stdin,
        help="Input file in fasta format (reads stdin if missing)")

    parser.add_argument(
        "-o", "--outfile", metavar='FILE', type=argparse.FileType('w'),
        default=sys.stdout,
        help="""output file for text (stdout by default, use -q/--quiet to suppress)""")

    parser.add_argument(
        "-q", "--quiet", dest="quiet", action='store_true', default=False,
        help="Suppress output of alignment to screen.")

    parser.add_argument(
        "--empty-ok", action='store_true', default=False,
        help="Exit with zero status if infile contains no sequences")

    # layout
    layout_options = parser.add_argument_group('Layout (applies to text and pdf)')

    layout_options.add_argument(
        "-w", "--width", dest="ncol", metavar="NUMBER", type=int, default=115,
        help="""Width of sequence to display in each block 'in
        characters [%(default)s]""")

    layout_options.add_argument(
        "-L", "--lines-per-block", dest="nrow", metavar="NUMBER", type=int, default=75,
        help="Sequences (lines) per block. [%(default)s]")

    # columns
    column_options = parser.add_argument_group('Column selection')

    column_options.add_argument(
        "-x", "--exclude-invariant",
        action='store_true', default=False,
        help="""Show only columns with at least N non-consensus bases
        (set N using '-a/--min-subs')""")

    column_options.add_argument(
        "-g", "--include-gapcols",
        action='store_true', default=False,
        help="Show columns containing only gap characters.")

    column_options.add_argument(
        "-r", "--range", dest="rawrange", metavar='INTERVAL',
        help="Range of columns to display (eg '-r start,stop')")

    column_options.add_argument(
        "-R", "--trim-to", metavar='SEQUENCE',
        help="""Trim alignment to the extent of this sequence
        identified by name or 1-based index (use
        `-i/--number-sequences` to display line numbers). """)

    column_options.add_argument(
        "-s", "--min-subs", dest="min_subs", metavar="NUMBER", type=int, default=1,
        help="""Minimum NUMBER of substitutions required to define a
        position as variable. [%(default)s]""")

    # consensus
    consensus_options = parser.add_argument_group(
        'Consensus display and sequence appearance')

    consensus_options.add_argument(
        "-c", "--consensus", dest="add_consensus", action='store_true', default=False,
        help="Show the consensus sequence [%(default)s]")

    consensus_options.add_argument(
        "-d", "--compare-to", dest="compare_to", metavar='SEQUENCE',
        help="""Identify the reference sequence by name or 1-based
        index (use `-i/--number-sequences` to display line
        numbers). Nucleotide positions identical to the reference will
        be replaced with `--simchar`. The default behavior is to use
        the consensus sequence as a reference.""")

    consensus_options.add_argument(
        "-D", "--no-comparison", action='store_false', dest='compare',
        default=True,
        help='Show all bases (ie, suppress comparsion with the reference sequence).')

    consensus_options.add_argument(
        "-G", "--ignore-gaps", dest="include_gaps", action="store_false",
        default=True, help='Ignore gaps in the calculation of a consensus.')

    consensus_options.add_argument(
        "-C", "--simchar", metavar='CHARACTER', default=None,
        help="""Character representing a base identical to the
        consensus. The default behavior is to identify differences
        using lower case.""")

    consensus_options.add_argument(
        '-t', '--reference-top', action='store_true',
        default=False,
        help="Place the reference/consensus sequence on the top of the alignment")

    # annotation
    name_options = parser.add_argument_group('Sequence annotation')

    name_options.add_argument(
        "-i", "--number-sequences", dest="seqnums", action='store_true', default=False,
        help="Show sequence number to left of name.")

    name_options.add_argument(
        "-n", "--name-max", dest="name_max", metavar="NUMBER", type=int, default=35,
        help="""Maximum width of sequence name in characters [%(default)s]""")

    name_options.add_argument(
        "-N", "--name-split", dest="name_split", metavar="CHARACTER",
        help="""Specify a character delimiting sequence names. By
        default, the name of each sequence is the first
        whitespace-delimited word. '--name-split=none' causes the
        entire line after the '>' to be displayed.""")

    name_options.add_argument(
        "-S", "--sort-by-name", type=argparse.FileType('r'), metavar='FILE',
        help="""File containing sequence names defining the sort-order
        of the sequences in the alignment.""")

    name_options.add_argument(
        "--rename-from-file", type=argparse.FileType('r'), metavar='FILE',
        help="""headerless csv file with columns 'old-name','new-name'
        to use for renaming the input sequences. If provided, renaming
        occurs immediately after reading the input sequences.""")

    if util.treeorder:
        name_options.add_argument(
            "-T", "--sort-by-tree", type=argparse.FileType('r'), metavar='FILE',
            help="""File containing a newick-format tree defining the
            sort-order of the sequences in the alignment (requires
            biopython).""")

    # for html output
    html_options = parser.add_argument_group('HTML output')
    html_options.add_argument(
        '--html', metavar='FILE', help="HTML output file")

    if html.default_brewer:
        html_options.add_argument(
            '--annotation-file', type=argparse.FileType('r'), metavar='FILE',
            help="""csv file with headers ("group", "col") specifying
            columns that should be colored in the html output. Each row
            identifies a label (group) and a corresponding column
            (1-indexed). Requires installation of the 'colorbrewer'
            package, which is python2 only.""")

    html_options.add_argument(
        '--table-only', action='store_true', default=False,
        help="""Don't produce a full html document, just the
        alignment table and style tags. Handy if you'd like to
        include in another document.""")

    html_options.add_argument(
        '--color', action='store_true', default=False,
        help='color nucleotides in output using default palette')

    html_options.add_argument(
        '--char-colors', type=argparse.FileType('r'), metavar='FILE',
        help="""csv file containing mapping of characters to
        HTML-defined colors (implies --color).""")

    html_options.add_argument(
        "--fontsize-html", metavar="NUMBER", default=7, type=int,
        help="Font size for html output [%(default)s]")

    # pdf options (only if reportlab is installed)
    if pdf.print_pdf:
        pdf_options = parser.add_argument_group(
            'PDF output',
            'These options require reportlab.')

        pdf_options.add_argument(
            '--pdf', metavar='FILE', help="PDF output file")

        pdf_options.add_argument(
            "-O", "--orientation", default='portrait',
            choices=('portrait', 'landscape'),
            help="Set page orientation; choose from portrait, landscape [%(default)s]")

        pdf_options.add_argument(
            "-b", "--blocks-per-page",
            metavar="NUMBER", type=int, default=1,
            help="Number of aligned blocks of sequence per page [%(default)s]")

        pdf_options.add_argument(
            "--fontsize-pdf", metavar="NUMBER", default=7, type=int,
            help="Font size for pdf output [%(default)s]")

    args = parser.parse_args(arguments or sys.argv[1:])

    # Ignore SIGPIPE, for head support
    exit_on_sigpipe()
    exit_on_sigint()

    seqs = util.readfasta(args.infile, name_split=args.name_split)

    if not seqs:
        if not args.quiet:
            print('No sequences in input')
        return 0 if args.empty_ok else 1

    if args.sort_by_name:
        sortdict = dict((line.strip(), i)
                        for i, line in enumerate(args.sort_by_name))
    elif getattr(args, 'sort_by_tree', None):
        names = util.treeorder(args.sort_by_tree)
        sortdict = dict((line.strip(), i) for i, line in enumerate(names))
    else:
        sortdict = {}

    if sortdict:
        seqs = iter(sorted(seqs, key=lambda seq: (sortdict.get(seq.name), seq.name)))

    if args.rename_from_file:
        namedict = dict(row for row in csv.reader(args.rename_from_file))
        seqs = (util.Seq(name=namedict.get(seq.name, seq.name), seq=seq.seq)
                for seq in seqs)

    formatted_seqs, vnumstrs, mask = util.reformat(
        seqs,
        add_consensus=args.add_consensus,
        compare=args.compare,
        compare_to=args.compare_to,
        exclude_gapcols=not args.include_gapcols,
        exclude_invariant=args.exclude_invariant,
        min_subs=args.min_subs,
        simchar=args.simchar,
        count_gaps=args.include_gaps,
        seqrange=get_range(args.rawrange) if args.rawrange else None,
        trim_to=args.trim_to,
        reference_top=args.reference_top)

    if args.html:
        if html.default_brewer and args.annotation_file:
            annotations = html.AnnotationSet.from_mapping_file(
                args.annotation_file, mask)
        else:
            annotations = None

        if args.char_colors:
            char_colors = html.parse_character_color_file(args.char_colors)
        elif args.color:
            with open(package_data('na_colors.csv')) as f:
                char_colors = html.parse_character_color_file(f)
        else:
            char_colors = None

        html.print_html(formatted_seqs, vnumstrs, mask,
                        outfile=args.html,
                        annotations=annotations,
                        fontsize=args.fontsize_html,
                        seqnums=args.seqnums,
                        charcolors=char_colors,
                        tableonly=args.table_only)

    all_numstrs = args.exclude_invariant or not args.include_gapcols

    pages = util.pagify(formatted_seqs, vnumstrs,
                        nrow=args.nrow,
                        ncol=args.ncol,
                        name_min=10,
                        name_max=args.name_max,
                        seqnums=args.seqnums,
                        all_numstrs=all_numstrs)

    if not args.quiet:
        for page in pages:
            for line in page:
                args.outfile.write(line.rstrip() + '\n')
            args.outfile.write('\n')

    if args.pdf:
        pdf.print_pdf(
            pages,
            outfile=args.pdf,
            fontsize=args.fontsize_pdf,
            orientation=args.orientation,
            blocks_per_page=args.blocks_per_page
        )

    for name, arg in args.__dict__.items():
        if hasattr(arg, 'close') and arg not in {sys.stdin, sys.stdout}:
            arg.close()


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import os
from setuptools import setup

from alnvu import __version__

# Get the long description from the README file
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    long_description = f.read()


params = {
    'author': 'Noah Hoffman',
    'author_email': 'noah.hoffman@gmail.com',
    'description': ('Reformat and condense multiple sequence alignments '
                    'to highlight variability'),
    'long_description': long_description,
    'name': 'alnvu',
    'package_dir': {'alnvu': 'alnvu'},
    'packages': ['alnvu'],
    'package_data': {'alnvu': ['data/*']},
    'url': 'http://github.com/nhoffman/alnvu',
    'version': __version__,
    'requires': ['python (>= 2.7)'],
    'install_requires': [
        'Jinja2>=2.7',
        'reportlab>=3.0',
        'fastalite>=0.3',
    ],
    'entry_points': {
        'console_scripts': ['av = alnvu.av:main']
    },
    'test_suite': 'tests',
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
}

setup(**params)



import traceback
import subprocess
import os
import sys


def get_version(datadir=None):
    """When executed from setup.py, creates the file "{datadir}/ver"
    containing a version number corresponding to the output of 'git
    describe --tags --dirty'. In all cases, the contents of this file
    is converted into a (hopefully) PEP-440 compliant version string
    and returned. ``datadir`` is an optional path to package data;
    when not provided, assumes package data resides in a directory
    named "data" in the same directory as this file (which in turn is
    expected to be placed in the same directory as the top-level
    package __init__.py). Further notes and assumptions:

    - get_version() should be called from the package __init__.py, for example:
        from ._version import get_version
        __version__ = get_version()
    - at least one git tag is defined
    - setup.py should import
    - setuptools.setup includes a directive to include {datadir}/ver in a
      package distribution, for example
      ``setuptools.setup(package_data={'my_package': ['data/*']})``

    """

    datadir = datadir or os.path.join(os.path.dirname(__file__), 'data')
    version_file = os.path.join(datadir, 'ver')

    # only try to create the version file if setup.py is someplace in the stack
    stack = traceback.extract_stack()
    try:
        in_setup = any(s.filename.endswith('setup.py') for s in stack)
    except AttributeError:
        in_setup = any(s[0].endswith('setup.py') for s in stack)

    if in_setup:
        sys.stdout.write('updating {} with version '.format(version_file))
        subprocess.call(
            ('mkdir -p {datadir} && '
             'git describe --tags --dirty > {file}.tmp '
             '&& mv {file}.tmp {file} '
             '|| rm -f {file}.tmp').format(datadir=datadir, file=version_file),
            shell=True, stderr=open(os.devnull, "w"))

    try:
        with open(version_file) as f:
            version = f.read().strip().replace('-', '+', 1).replace('-', '.')
    except Exception:
        version = ''

    if in_setup:
        sys.stdout.write(version + '\n')

    return version
